package bdd;


public class Functions {

	public static  final Function FALSE = new ConstantFunction(false);
	
	public static final Function TRUE = new ConstantFunction(true);
	
	public static class ConstantFunction implements Function
	{
		private boolean value;
		
		public ConstantFunction(boolean value)
		{
			this.value = value;
		}
		
		public Function eval(int var, boolean value) 
		{
			return this;
		}

		public boolean getValue() 
		{
			return value;
		}

		public int getMaxVar() 
		{
			return -1;
		}
	}
	
	public static class NotFunction implements Function
	{
		
		private Function f;
		
		public NotFunction(Function f)
		{
			this.f = f;
		}

		public Function eval(int var, boolean value) 
		{
			return new NotFunction(f.eval(var, value));
		}

		public boolean getValue() 
		{
			return !f.getValue();
		}

		public int getMaxVar() 
		{
			return f.getMaxVar();
		}
		
	}
	
	public static class AndFunction implements Function
	{
		private Function f1;
		private Function f2;
		
		public AndFunction(Function f1, Function f2)
		{
			this.f1 = f1;
			this.f2 = f2;
		}

		public Function eval(int var, boolean value) 
		{
			return new AndFunction(f1.eval(var, value), f2.eval(var, value));
		}

		public boolean getValue() 
		{
			return f1.getValue() && f2.getValue();
		}

		public int getMaxVar() 
		{
			return Math.max(f1.getMaxVar(), f2.getMaxVar());
		}
	}
	
	public static class OrFunction implements Function
	{
		private Function f1;
		private Function f2;
		
		public OrFunction(Function f1, Function f2)
		{
			this.f1 = f1;
			this.f2 = f2;
		}

		public Function eval(int var, boolean value) 
		{
			return new OrFunction(f1.eval(var, value), f2.eval(var, value));
		}

		public boolean getValue() 
		{
			return f1.getValue() || f2.getValue();
		}

		public int getMaxVar() 
		{
			return  Math.max(f1.getMaxVar(), f2.getMaxVar());
		}
	}
	
	public static  class VarFunction implements Function
	{
		private int var;

		public VarFunction(int var)
		{
			this.var = var;
		}
		
		public Function eval(int var, boolean value) 
		{
			if (this.var == var) {
				return (value) ? TRUE : FALSE;
			}
			else {
				return new VarFunction(this.var);
			}
		}

		public boolean getValue() 
		{
			throw new IllegalStateException("Not evaluated yet " + var);
		}

		public int getMaxVar() 
		{
			return var;
		}
		
	}
	
}
